@@ -69,13 +69,13 @@ struct _EmerillonSidebarPrivate
     GtkTreeModel *page_model;
 };
 
-G_DEFINE_TYPE (EmerillonSidebar, emerillon_sidebar, GTK_TYPE_VBOX)
+G_DEFINE_TYPE (EmerillonSidebar, emerillon_sidebar, GTK_TYPE_BOX)
 
 #define EMERILLON_SIDEBAR_GET_PRIVATE(object) \
     (G_TYPE_INSTANCE_GET_PRIVATE ((object), EMERILLON_TYPE_SIDEBAR, EmerillonSidebarPrivate))
 
 static void
-emerillon_sidebar_destroy (GtkObject *object)
+emerillon_sidebar_dispose (GObject *object)
 {
   EmerillonSidebar *sidebar = EMERILLON_SIDEBAR (object);
 
@@ -91,7 +91,7 @@ emerillon_sidebar_destroy (GtkObject *object)
       sidebar->priv->page_model = NULL;
     }
 
-  (* GTK_OBJECT_CLASS (emerillon_sidebar_parent_class)->destroy) (object);
+  (* G_OBJECT_CLASS (emerillon_sidebar_parent_class)->dispose) (object);
 }
 
 static void
@@ -195,16 +195,12 @@ static void
 emerillon_sidebar_class_init (EmerillonSidebarClass *emerillon_sidebar_class)
 {
   GObjectClass *g_object_class;
-  GtkWidgetClass *widget_class;
-  GtkObjectClass *gtk_object_klass;
 
   g_object_class = G_OBJECT_CLASS (emerillon_sidebar_class);
-  widget_class = GTK_WIDGET_CLASS (emerillon_sidebar_class);
-  gtk_object_klass = GTK_OBJECT_CLASS (emerillon_sidebar_class);
 
   g_type_class_add_private (g_object_class, sizeof (EmerillonSidebarPrivate));
 
-  gtk_object_klass->destroy = emerillon_sidebar_destroy;
+  g_object_class->dispose = emerillon_sidebar_dispose;
   g_object_class->get_property = emerillon_sidebar_get_property;
   g_object_class->set_property = emerillon_sidebar_set_property;
 
@@ -239,14 +235,16 @@ emerillon_sidebar_menu_position_under (GtkMenu *menu,
   GtkWidget *widget;
 
   g_return_if_fail (GTK_IS_BUTTON (user_data));
-  g_return_if_fail (GTK_WIDGET_NO_WINDOW (user_data));
-
+  g_return_if_fail (!gtk_widget_get_has_window (GTK_WIDGET (user_data)));
+  
   widget = GTK_WIDGET (user_data);
 
-  gdk_window_get_origin (widget->window, x, y);
+  gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
 
-  *x += widget->allocation.x;
-  *y += widget->allocation.y + widget->allocation.height;
+  GtkAllocation allocation;
+  gtk_widget_get_allocation (widget, &allocation);
+  *x += allocation.x;
+  *y += allocation.y + allocation.height;
 
   *push_in = FALSE;
 }
@@ -260,15 +258,18 @@ emerillon_sidebar_select_button_press_cb (GtkWidget *widget,
 
   if (event->button == 1)
     {
-      GtkRequisition requisition;
+      gint preferred_width;
       gint width;
 
-      width = widget->allocation.width;
+      GtkAllocation allocation;
+      gtk_widget_get_allocation (widget, &allocation);
+      width = allocation.width;
 
       gtk_widget_set_size_request (sidebar->priv->menu, -1, -1);
-      gtk_widget_size_request (sidebar->priv->menu, &requisition);
+      gtk_widget_get_preferred_width(sidebar->priv->menu, NULL, 
+          &preferred_width);
       gtk_widget_set_size_request (sidebar->priv->menu,
-          MAX (width, requisition.width), -1);
+          MAX (width, preferred_width), -1);
 
       gtk_widget_grab_focus (widget);
 
@@ -291,10 +292,10 @@ emerillon_sidebar_select_button_key_press_cb (GtkWidget *widget,
 {
   EmerillonSidebar *sidebar = EMERILLON_SIDEBAR (user_data);
 
-  if (event->keyval == GDK_space ||
-      event->keyval == GDK_KP_Space ||
-      event->keyval == GDK_Return ||
-      event->keyval == GDK_KP_Enter)
+  if (event->keyval == GDK_KEY_space ||
+      event->keyval == GDK_KEY_KP_Space ||
+      event->keyval == GDK_KEY_Return ||
+      event->keyval == GDK_KEY_KP_Enter)
     {
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 
@@ -382,6 +383,9 @@ emerillon_sidebar_init (EmerillonSidebar *sidebar)
 
   sidebar->priv = EMERILLON_SIDEBAR_GET_PRIVATE (sidebar);
 
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (sidebar),
+      GTK_ORIENTATION_VERTICAL);
+
   /* data model */
   sidebar->priv->page_model = (GtkTreeModel *) gtk_list_store_new (
       PAGE_COLUMN_NUM_COLS,
@@ -391,7 +395,7 @@ emerillon_sidebar_init (EmerillonSidebar *sidebar)
       G_TYPE_INT);
 
   /* top option menu */
-  hbox = gtk_hbox_new (FALSE, 0);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   sidebar->priv->hbox = hbox;
   gtk_box_pack_start (GTK_BOX (sidebar), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
@@ -399,6 +403,7 @@ emerillon_sidebar_init (EmerillonSidebar *sidebar)
   sidebar->priv->select_button = gtk_toggle_button_new ();
   gtk_button_set_relief (GTK_BUTTON (sidebar->priv->select_button), 
       GTK_RELIEF_NONE);
+  gtk_widget_set_sensitive (sidebar->priv->select_button, FALSE);
 
   g_signal_connect (sidebar->priv->select_button, "button_press_event",
       G_CALLBACK (emerillon_sidebar_select_button_press_cb),
@@ -408,13 +413,17 @@ emerillon_sidebar_init (EmerillonSidebar *sidebar)
       G_CALLBACK (emerillon_sidebar_select_button_key_press_cb),
       sidebar);
 
-  select_hbox = gtk_hbox_new (FALSE, 0);
+  select_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
   sidebar->priv->label = gtk_label_new ("");
+  gtk_label_set_ellipsize (GTK_LABEL (sidebar->priv->label),
+                           PANGO_ELLIPSIZE_END);
+  gtk_misc_set_alignment (GTK_MISC (sidebar->priv->label),
+                          0, 0.5);
 
   gtk_box_pack_start (GTK_BOX (select_hbox),
       sidebar->priv->label,
-      FALSE, FALSE, 0);
+      TRUE, TRUE, 0);
 
   gtk_widget_show (sidebar->priv->label);
 
@@ -477,6 +486,16 @@ emerillon_sidebar_new (void)
   return sidebar;
 }
 
+/**
+ * emerillon_sidebar_add_page:
+ * @sidebar: (in): An #EmerillonSidebar instance
+ * @title: (in): Sidebar title
+ * @main_widget: (in): A #GtkWidget which should be embedded in the sidebar
+ *
+ * Adds a new page to the sidebar
+ *
+ * See: emerillon_window_get_sidebar()
+ */
 void
 emerillon_sidebar_add_page (EmerillonSidebar *sidebar,
                              const gchar *title,
@@ -534,6 +553,9 @@ emerillon_sidebar_add_page (EmerillonSidebar *sidebar,
 
   g_free (label_title);
 
+  if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (sidebar->priv->notebook)) > 1)
+    gtk_widget_set_sensitive (sidebar->priv->select_button, TRUE);
+
   g_signal_emit (G_OBJECT (sidebar), signals[SIGNAL_PAGE_ADDED],
       0, main_widget);
 }
@@ -545,6 +567,7 @@ emerillon_sidebar_remove_page (EmerillonSidebar *sidebar,
   GtkTreeIter iter;
   GtkWidget *widget, *menu_item;
   gboolean valid;
+  gboolean visible;
   gint index;
 
   g_return_if_fail (EMERILLON_IS_SIDEBAR (sidebar));
@@ -571,6 +594,9 @@ emerillon_sidebar_remove_page (EmerillonSidebar *sidebar,
 
   if (valid)
     {
+      visible = (index == gtk_notebook_get_current_page (
+          GTK_NOTEBOOK (sidebar->priv->notebook)));
+
       gtk_notebook_remove_page (GTK_NOTEBOOK (sidebar->priv->notebook), 
           index);
 
@@ -579,11 +605,36 @@ emerillon_sidebar_remove_page (EmerillonSidebar *sidebar,
       gtk_list_store_remove (GTK_LIST_STORE (sidebar->priv->page_model), 
           &iter);
 
-      g_signal_emit (G_OBJECT (sidebar), 
+      /* If we removed the visible tab, we select the first page or remove
+       * the text in the dropdown menu if no page is left */
+      if (visible)
+        {
+          if (!emerillon_sidebar_is_empty(sidebar))
+            {
+              valid = gtk_tree_model_get_iter_first (sidebar->priv->page_model, &iter);
+              emerillon_sidebar_select_page (sidebar, &iter);
+            }
+          else
+            gtk_label_set_text (GTK_LABEL (sidebar->priv->label), "");
+        }
+
+      if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (sidebar->priv->notebook)) <= 1)
+        gtk_widget_set_sensitive (sidebar->priv->select_button, FALSE);
+
+      g_signal_emit (G_OBJECT (sidebar),
           signals[SIGNAL_PAGE_REMOVED], 0, main_widget);
     }
 }
 
+/**
+ * emerillon_sidebar_get_n_pages:
+ * @sidebar: (in): An #EmerillonSidebar instance
+ *
+ * Returns the number of pages @sidebar has.
+ *
+ * Return value: The number of pages @sidebar has. If @sidebar isn't
+ * an #EmerillonSidebar instance this function returns #TRUE.
+ */
 gint
 emerillon_sidebar_get_n_pages (EmerillonSidebar *sidebar)
 {
@@ -593,6 +644,15 @@ emerillon_sidebar_get_n_pages (EmerillonSidebar *sidebar)
       GTK_TREE_MODEL (sidebar->priv->page_model), NULL);
 }
 
+/**
+ * emerillon_sidebar_is_empty:
+ * @sidebar: (in): An #EmerillonSidebar instance
+ *
+ * Checks if the @sidebar is empty
+ *
+ * Return value: #TRUE if @sidebar has no pages or isn't
+ * an #EmerillonSidebar instance
+ */
 gboolean
 emerillon_sidebar_is_empty (EmerillonSidebar *sidebar)
 {
