@@ -21,7 +21,7 @@
 #include "config.h"
 #endif
 
-#include "placemarks.h"
+#include "cut-paste/totem-plugin.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -32,9 +32,14 @@
 #include "manage-dialog.h"
 #include "placemarks-model.h"
 
-G_DEFINE_TYPE (PlacemarksPlugin, placemarks_plugin, ETHOS_TYPE_PLUGIN)
+#define PLACEMARKS_TYPE_PLUGIN            (placemarks_plugin_get_type())
+#define PLACEMARKS_PLUGIN(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PLACEMARKS_TYPE_PLUGIN, PlacemarksPlugin))
+#define PLACEMARKS_PLUGIN_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  PLACEMARKS_TYPE_PLUGIN, PlacemarksPluginClass))
+#define PLACEMARKS_IS_PLUGIN(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PLACEMARKS_TYPE_PLUGIN))
+#define PLACEMARKS_IS_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  PLACEMARKS_TYPE_PLUGIN))
+#define PLACEMARKS_PLUGIN_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  PLACEMARKS_TYPE_PLUGIN, PlacemarksPluginClass))
 
-struct _PlacemarksPluginPrivate
+typedef struct
 {
   EmerillonWindow *window;
   ChamplainView *map_view;
@@ -47,9 +52,16 @@ struct _PlacemarksPluginPrivate
   GtkTreeModel *model;
   GtkWidget *menu;
 
+  /** Layer with placemark markers */
+  ChamplainMarkerLayer *markers_layer;
   guint deleted_cb_id;
-};
+} PlacemarksPluginPrivate;
 
+TOTEM_PLUGIN_REGISTER (PLACEMARKS_TYPE_PLUGIN, PlacemarksPlugin, placemarks_plugin);
+
+/**
+ * Go to the specified placemark
+ */
 static void
 go_cb (GtkAction *action,
        PlacemarksPlugin *plugin)
@@ -95,6 +107,26 @@ go_cb (GtkAction *action,
   champlain_view_center_on (priv->map_view, lat, lon);
 }
 
+static ChamplainMarker *
+add_marker(PlacemarksPlugin *plugin, const gchar *name, gdouble lat, gdouble lon)
+{
+  PlacemarksPluginPrivate *priv;
+  priv = PLACEMARKS_PLUGIN (plugin)->priv;
+  ChamplainMarker *marker;
+
+  ClutterColor orange = { 0xf3, 0x94, 0x07, 0xbb };
+  marker = CHAMPLAIN_MARKER ( champlain_label_new_with_text (name, "Serif 14",
+                                                              NULL, NULL));
+  champlain_label_set_use_markup (CHAMPLAIN_LABEL (marker), TRUE);
+  champlain_label_set_alignment (CHAMPLAIN_LABEL (marker), PANGO_ALIGN_RIGHT);
+  champlain_label_set_color (CHAMPLAIN_LABEL (marker), &orange);
+
+  champlain_location_set_location (CHAMPLAIN_LOCATION(marker), lat, lon);
+  champlain_marker_layer_add_marker (priv->markers_layer, CHAMPLAIN_MARKER (marker));
+
+  return marker;
+}
+
 static guint
 append_menu_item (PlacemarksPlugin *plugin,
                   const gchar *id,
@@ -209,6 +241,8 @@ save_placemarks (PlacemarksPlugin *plugin)
     }
 
   g_key_file_free (file);
+  g_free (data);
+  g_free (filename);
   return FALSE;
 }
 
@@ -241,12 +275,14 @@ add_placemark (PlacemarksPlugin *plugin,
   gchar *lat_str, *lon_str, *zoom_str;
   GtkTreeIter iter;
   PlacemarksPluginPrivate *priv;
+  ChamplainMarker *marker;
 
   priv = PLACEMARKS_PLUGIN (plugin)->priv;
 
   lat_str = g_strdup_printf ("%f", lat);
   lon_str = g_strdup_printf ("%f", lon);
   zoom_str = g_strdup_printf ("%d", zoom);
+  marker = add_marker (plugin, name, lat, lon);
 
   gtk_list_store_append (GTK_LIST_STORE (priv->model), &iter);
   gtk_list_store_set (GTK_LIST_STORE (priv->model), &iter,
@@ -258,6 +294,7 @@ add_placemark (PlacemarksPlugin *plugin,
                       COL_LON_STR, lon_str,
                       COL_ZOOM, zoom,
                       COL_ZOOM_STR, zoom_str,
+                      COL_MARKER, marker,
                       -1);
 
   g_free (lat_str);
@@ -368,6 +405,7 @@ load_placemarks (PlacemarksPlugin *plugin)
   groups = g_key_file_get_groups (file, &group_count);
   priv->placemark_count = group_count;
 
+
   for (i = 0; i < group_count; i++)
     {
       gchar *name;
@@ -519,7 +557,7 @@ row_deleted_cb (GtkTreeModel *tree_model,
 }
 
 static void
-activated (EthosPlugin *plugin)
+impl_activate (PeasActivatable *plugin)
 {
   PlacemarksPluginPrivate *priv;
   GtkUIManager *manager;
@@ -529,6 +567,9 @@ activated (EthosPlugin *plugin)
   priv->window = EMERILLON_WINDOW (emerillon_window_dup_default ());
   priv->map_view = emerillon_window_get_map_view (priv->window);
 
+  priv->markers_layer = champlain_marker_layer_new_full (CHAMPLAIN_SELECTION_NONE);
+  champlain_view_add_layer(priv->map_view, CHAMPLAIN_LAYER(priv->markers_layer));
+
   manager = emerillon_window_get_ui_manager (priv->window);
 
   priv->action_group = gtk_action_group_new ("PlacemarksActions");
@@ -556,7 +597,8 @@ activated (EthosPlugin *plugin)
                               G_TYPE_STRING,       /* Longitude as a string */
                               G_TYPE_INT,          /* Zoom level */
                               G_TYPE_STRING,       /* Zoom level as a string */
-                              G_TYPE_UINT);        /* UI ID */
+                              G_TYPE_UINT,        /* UI ID */
+                              G_TYPE_POINTER);       /* Marker pointer */
   priv->model = GTK_TREE_MODEL (store);
   priv->deleted_cb_id  =  g_signal_connect (priv->model,
                                             "row-deleted",
@@ -568,7 +610,7 @@ activated (EthosPlugin *plugin)
 }
 
 static void
-deactivated (EthosPlugin *plugin)
+impl_deactivate (PeasActivatable *plugin)
 {
   GtkUIManager *manager;
   PlacemarksPluginPrivate *priv;
@@ -584,37 +626,3 @@ deactivated (EthosPlugin *plugin)
 
   g_object_unref (priv->model);
 }
-
-static void
-placemarks_plugin_class_init (PlacemarksPluginClass *klass)
-{
-  EthosPluginClass *plugin_class;
-
-  g_type_class_add_private (klass, sizeof (PlacemarksPluginPrivate));
-
-  plugin_class = ETHOS_PLUGIN_CLASS (klass);
-  plugin_class->activated = activated;
-  plugin_class->deactivated = deactivated;
-}
-
-static void
-placemarks_plugin_init (PlacemarksPlugin *plugin)
-{
-  plugin->priv = G_TYPE_INSTANCE_GET_PRIVATE (plugin,
-                                              PLACEMARKS_TYPE_PLUGIN,
-                                              PlacemarksPluginPrivate);
-  plugin->priv->placemark_count = 0;
-  plugin->priv->deleted_cb_id = 0;
-}
-
-EthosPlugin*
-placemarks_plugin_new (void)
-{
-  return g_object_new (PLACEMARKS_TYPE_PLUGIN, NULL);
-}
-
-G_MODULE_EXPORT EthosPlugin*
-ethos_plugin_register (void)
-{
-  return placemarks_plugin_new ();
-}
